@@ -47,19 +47,6 @@
 #include "nv-retpoline.h"
 #endif
 
-#if defined(NVCPU_X86_64) && defined(CONFIG_IA32_EMULATION) && \
-  !defined(NV_FILE_OPERATIONS_HAS_COMPAT_IOCTL)
-#  define NV_NEEDS_COMPAT_IOCTL_REGISTRATION 1
-#else
-#  define NV_NEEDS_COMPAT_IOCTL_REGISTRATION 0
-#endif
-
-
-#if NV_NEEDS_COMPAT_IOCTL_REGISTRATION
-#include <linux/syscalls.h> /* sys_ioctl() */
-#include <linux/ioctl32.h> /* register_ioctl32_conversion() */
-#endif
-
 #define NVKMS_MAJOR_DEVICE_NUMBER 195
 #define NVKMS_MINOR_DEVICE_NUMBER 254
 
@@ -1265,20 +1252,6 @@ static int nvkms_ioctl(struct inode *inode, struct file *filp,
     return status;
 }
 
-static void nvkms_register_compatible_ioctl(void)
-{
-#if NV_NEEDS_COMPAT_IOCTL_REGISTRATION
-    register_ioctl32_conversion(NVKMS_IOCTL_IOWR, (void *)sys_ioctl);
-#endif
-}
-
-static void nvkms_unregister_compatible_ioctl(void)
-{
-#if NV_NEEDS_COMPAT_IOCTL_REGISTRATION
-    unregister_ioctl32_conversion(NVKMS_IOCTL_IOWR);
-#endif
-}
-
 static unsigned int nvkms_poll(struct file *filp, poll_table *wait)
 {
     unsigned int mask = 0;
@@ -1354,8 +1327,6 @@ static int __init nvkms_init(void)
         return ret;
     }
 
-    nvkms_register_compatible_ioctl();
-
     down(&nvkms_lock);
     nvKmsModuleLoad();
     up(&nvkms_lock);
@@ -1419,8 +1390,6 @@ restart:
     nv_kthread_q_stop(&nvkms_deferred_close_kthread_q);
     nv_kthread_q_stop(&nvkms_kthread_q);
 
-    nvkms_unregister_compatible_ioctl();
-
     nvidia_unregister_module(&nvidia_modeset_module);
     nvkms_free_rm();
 }
