@@ -57,17 +57,17 @@ static NV_STATUS uvm8_test_get_gpu_ref_count(UVM_TEST_GET_GPU_REF_COUNT_PARAMS *
 static NV_STATUS uvm8_test_peer_ref_count(UVM_TEST_PEER_REF_COUNT_PARAMS *params, struct file *filp)
 {
     NvU64 registered_ref_count = 0;
-    uvm_gpu_t *gpu_1 = NULL;
-    uvm_gpu_t *gpu_2 = NULL;
+    uvm_gpu_t *gpu0 = NULL;
+    uvm_gpu_t *gpu1 = NULL;
     NV_STATUS status = NV_OK;
 
     uvm_mutex_lock(&g_uvm_global.global_lock);
 
-    gpu_1 = uvm_gpu_get_by_uuid(&params->gpu_uuid_1);
-    gpu_2 = uvm_gpu_get_by_uuid(&params->gpu_uuid_2);
+    gpu0 = uvm_gpu_get_by_uuid(&params->gpu_uuid_1);
+    gpu1 = uvm_gpu_get_by_uuid(&params->gpu_uuid_2);
 
-    if (gpu_1 != NULL && gpu_2 != NULL) {
-        uvm_gpu_peer_t *peer_caps = uvm_gpu_peer_caps(gpu_1, gpu_2);
+    if (gpu0 != NULL && gpu1 != NULL) {
+        uvm_gpu_peer_t *peer_caps = uvm_gpu_peer_caps(gpu0, gpu1);
         registered_ref_count = peer_caps->ref_count;
     }
     else {
@@ -105,6 +105,87 @@ static NV_STATUS uvm8_test_nv_kthread_q(
     return NV_ERR_INVALID_STATE;
 }
 
+static NV_STATUS uvm8_test_numa_get_closest_cpu_node_to_gpu(UVM_TEST_NUMA_GET_CLOSEST_CPU_NODE_TO_GPU_PARAMS *params,
+                                                           struct file *filp)
+{
+    uvm_gpu_t *gpu;
+    NV_STATUS status;
+
+    status = uvm_gpu_retain_by_uuid(&params->gpu_uuid, &gpu);
+    if (status != NV_OK)
+        return status;
+
+    params->node_id = gpu->closest_cpu_numa_node;
+    uvm_gpu_release(gpu);
+    return NV_OK;
+}
+
+// Callers of this function should ensure that node is not NUMA_NO_NODE in order to avoid
+// overrunning the kernel's node to cpumask map.
+static NV_STATUS uvm8_test_verify_bh_affinity(uvm_intr_handler_t *isr, int node)
+{
+    UVM_ASSERT(node != NUMA_NO_NODE);
+
+    // If the bottom half has been executed but its cpus_used_mask is empty
+    // something obviously went wrong. Otherwise, check that the CPUs on which
+    // the bottom half was executed is a subset of the NUMA node's cpumask.
+    if ((isr->stats.bottom_half_count && cpumask_empty(&isr->stats.cpus_used_mask)) ||
+        !cpumask_subset(&isr->stats.cpus_used_mask, uvm_cpumask_of_node(node))) {
+        UVM_TEST_PRINT("ISR BH cpu mask check failed! BH ran on CPU cores outside NUMA %u\n",
+                       node);
+        return NV_ERR_INVALID_STATE;
+    }
+    return NV_OK;
+}
+
+static NV_STATUS uvm8_test_numa_check_affinity(UVM_TEST_NUMA_CHECK_AFFINITY_PARAMS *params,
+                                               struct file *filp)
+{
+    uvm_gpu_t *gpu;
+    NV_STATUS status;
+
+    if (!UVM_THREAD_AFFINITY_SUPPORTED())
+        return NV_ERR_NOT_SUPPORTED;
+
+    status = uvm_gpu_retain_by_uuid(&params->gpu_uuid, &gpu);
+    if (status != NV_OK)
+        return status;
+
+    // If the GPU is not attached to a NUMA node, there is nothing to do.
+    if (gpu->closest_cpu_numa_node == NUMA_NO_NODE) {
+        status = NV_ERR_NOT_SUPPORTED;
+        goto release;
+    }
+
+    if (gpu->replayable_faults_supported) {
+        uvm_gpu_replayable_faults_isr_lock(gpu);
+        status = uvm8_test_verify_bh_affinity(&gpu->isr.replayable_faults,
+                                              gpu->closest_cpu_numa_node);
+        uvm_gpu_replayable_faults_isr_unlock(gpu);
+        if (status != NV_OK)
+            goto release;
+
+        if (gpu->non_replayable_faults_supported) {
+            uvm_gpu_non_replayable_faults_isr_lock(gpu);
+            status = uvm8_test_verify_bh_affinity(&gpu->isr.non_replayable_faults,
+                                                  gpu->closest_cpu_numa_node);
+            uvm_gpu_non_replayable_faults_isr_unlock(gpu);
+            if (status != NV_OK)
+                goto release;
+        }
+
+        if (gpu->access_counters_supported) {
+            uvm_gpu_access_counters_isr_lock(gpu);
+            status = uvm8_test_verify_bh_affinity(&gpu->isr.access_counters,
+                                                  gpu->closest_cpu_numa_node);
+            uvm_gpu_access_counters_isr_unlock(gpu);
+        }
+    }
+release:
+    uvm_gpu_release(gpu);
+    return status;
+}
+
 static NV_STATUS uvm8_test_get_kernel_virtual_address(
         UVM_TEST_GET_KERNEL_VIRTUAL_ADDRESS_PARAMS *params,
         struct file *filp)
@@ -198,6 +279,14 @@ long uvm8_test_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
         UVM_ROUTE_CMD_STACK(UVM_TEST_GET_GPU_TIME,                  uvm8_test_get_gpu_time);
         UVM_ROUTE_CMD_STACK(UVM_TEST_PMM_RELEASE_FREE_ROOT_CHUNKS,  uvm8_test_pmm_release_free_root_chunks);
         UVM_ROUTE_CMD_STACK(UVM_TEST_DRAIN_REPLAYABLE_FAULTS,       uvm8_test_drain_replayable_faults);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_PMA_GET_BATCH_SIZE,            uvm8_test_pma_get_batch_size);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_PMM_QUERY_PMA_STATS,           uvm8_test_pmm_query_pma_stats);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_NUMA_GET_CLOSEST_CPU_NODE_TO_GPU, uvm8_test_numa_get_closest_cpu_node_to_gpu);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_NUMA_CHECK_AFFINITY,           uvm8_test_numa_check_affinity);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_VA_SPACE_ADD_DUMMY_THREAD_CONTEXTS,    uvm8_test_va_space_add_dummy_thread_contexts);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_VA_SPACE_REMOVE_DUMMY_THREAD_CONTEXTS, uvm8_test_va_space_remove_dummy_thread_contexts);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_THREAD_CONTEXT_SANITY,         uvm8_test_thread_context_sanity);
+        UVM_ROUTE_CMD_STACK(UVM_TEST_THREAD_CONTEXT_PERF,           uvm8_test_thread_context_perf);
     }
 
     return -EINVAL;
